( cd usr/share/doc/HTML/nl/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/nl/kdebluetooth ; ln -sf /usr/share/doc/HTML/nl/common common )
( cd usr/share/doc/HTML/ca/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/ca/kdebluetooth ; ln -sf /usr/share/doc/HTML/ca/common common )
( cd usr/share/doc/HTML/fr/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/fr/kdebluetooth ; ln -sf /usr/share/doc/HTML/fr/common common )
( cd usr/share/doc/HTML/pt/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/pt/kdebluetooth ; ln -sf /usr/share/doc/HTML/pt/common common )
( cd usr/share/doc/HTML/en/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/en/kdebluetooth ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/et/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/et/kdebluetooth ; ln -sf /usr/share/doc/HTML/et/common common )
( cd usr/share/doc/HTML/sv/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/sv/kdebluetooth ; ln -sf /usr/share/doc/HTML/sv/common common )
( cd usr/share/doc/HTML/it/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/it/kdebluetooth ; ln -sf /usr/share/doc/HTML/it/common common )
( cd usr/share/doc/HTML/ru/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/ru/kdebluetooth ; ln -sf /usr/share/doc/HTML/ru/common common )
( cd usr/share/doc/HTML/es/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/es/kdebluetooth ; ln -sf /usr/share/doc/HTML/es/common common )
( cd usr/share/doc/HTML/da/kdebluetooth ; rm -rf common )
( cd usr/share/doc/HTML/da/kdebluetooth ; ln -sf /usr/share/doc/HTML/da/common common )
( cd usr/lib ; rm -rf libkbluetooth.so.0 )
( cd usr/lib ; ln -sf libkbluetooth.so.0.0.0 libkbluetooth.so.0 )
( cd usr/lib ; rm -rf libqobex.so )
( cd usr/lib ; ln -sf libqobex.so.0.0.9 libqobex.so )
( cd usr/lib ; rm -rf libqobex.so.0 )
( cd usr/lib ; ln -sf libqobex.so.0.0.9 libqobex.so.0 )
( cd usr/lib ; rm -rf libkbluetooth.so )
( cd usr/lib ; ln -sf libkbluetooth.so.0.0.0 libkbluetooth.so )
