<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kbotobexclient">
	<MenuBar> 
		<Menu name="settings">
			<Action name="view_device_selector" append="show_merge" />
			<Action name="view_file_selector" append="show_merge" />
			<Action name="view_file_browser" append="show_merge" />	
		</Menu> 
	</MenuBar>
</kpartgui>
